-------------------------------------------------------
-- Design Name : mfh_icm_tb_01
-- File Name   : mfh_icm_tb_01.vhd
-- Function    : UART transmitter test bench
-- Coder       : K.-H. Sulanke, DESY
-- Date        : 2019-11-21
-------------------------------------------------------
library ieee;
    use ieee.std_logic_1164.all;
    use ieee.numeric_std.all;
 
entity mfh_icm_tb_01 is
end mfh_icm_tb_01;

architecture behavior of mfh_icm_tb_01 is
  
component mfh_icm_top_01 is 
port(    
            -- ************************ Bank #14, VCCO = 1.8V  ****************************************************
      ICM_CLOCK_P     : in std_logic;  -- DIFF_SSTL18_I, clock
      ICM_CLOCK_N     : in std_logic;  --
      ICM_IRIGB_P     : in std_logic;  -- DIFF_SSTL18_I, 
      ICM_IRIGB_N     : in std_logic;  --
      ICM_IRIGB_LOC   : out std_logic;  -- artificially generated IRIGB code
       
      ICM_nMASTER     : in std_logic;  -- to select the ICM_IRIGB_LOC source
--      ICM_EXT_GPIN    : in std_logic;  -- external general purpose input
       
      WP_nPON         : out  std_logic;  --        
      WP_ADC_nCS0     : out  std_logic;  --  SPI interface, voltage & current measurement
      WP_ADC_nCS1     : out  std_logic;  -- 
      WP_ADC_SCLK     : out  std_logic;  -- common clock
      WP_ADC_DOUT0    : in   std_logic;  -- current
      WP_ADC_DOUT1    : in   std_logic;  -- voltaga   
    
      ZYNQ_TXD        : in  std_logic;  --  
      ZYNQ_RXD        : out std_logic;  --       
      ZYNQ_CTS        : out std_logic;  -- 
      ZYNQ_RTS        : in  std_logic;  --
                                          
      COM_ADC_D       : in  std_logic_vector(13 downto 0);    -- communication ADC output
      COM_ADC_CSBn    : out std_logic; --
      COM_ADC_SCLK    : out std_logic; --
      COM_ADC_SDIO    : inout std_logic; -- is inout
      COM_ADC_CLK_P   : out std_logic;  -- DIFF_SSTL18_I, comm. ADC  clock     
      COM_ADC_CLK_N   : out std_logic;  -- 

      D00_MOSI        : inout std_logic;  -- 
      D01_DIN         : inout std_logic;  -- 
      D02             : inout std_logic;  -- 
      D03             : inout std_logic;  -- 
      FLASH_CSn       : out std_logic;  -- 
      
      -- ************************** Bank #34, VCCO = 3.3V  ****************************************************
      QOSC_CLK        : in  std_logic;  -- 20 MHz local oscillator
       
      COM_DAC_DB      : out std_logic_vector(11 downto 0);    -- connected to communication DAC
      COM_DAC_CLOCK   : out std_logic;  -- 
      COM_DAC_SLEEP   : out std_logic;  -- 
    
      USB1_TXD        : in  std_logic;  --   
      USB1_RXD        : out std_logic;  --       
      USB1_CTS        : out std_logic;  -- 
      USB1_RTS        : in  std_logic;  --
      
      ICM_ID          : out std_logic; --inout
      nCBL_PLGD       : in std_logic;  -- with pullup

      nLED_ENA        : out std_logic; --   
      nTX_LED         : out std_logic; -- yellow, located on ICM (V12)
      nRX_LED         : out std_logic; -- green, located on ICM (V13)

      nTX_LED_EXT     : out std_logic; -- yellow, located at chassis
      nRX_LED_EXT     : out std_logic; -- green, located at chassis
 
      TEST_IO_0       : in  std_logic;
      TEST_IO_1       : out std_logic;
      TEST_IO_2       : out std_logic;
      TEST_IO_3       : in  std_logic;
      
      TERM_FPGA       : out std_logic  -- '1' activates cable termination       
     );
end component mfh_icm_top_01 ;

    signal ICM_CLOCK_P      : std_logic := '0'; 
    signal ICM_CLOCK_N      : std_logic := '1'; 
    signal ICM_IRIGB_P    : std_logic := '0'; 
    signal ICM_IRIGB_N    : std_logic := '1'; 
    signal ICM_IRIGB_LOC  : std_logic; 
    signal ICM_nMASTER    : std_logic := '0';    -- 3 to select between RS232/485, 4..5 are being used as ADR_SW
    signal WP_nPON        : std_logic := '0';   --
    signal WP_ADC_nCS0    : std_logic;   --
    signal WP_ADC_nCS1    : std_logic;   --
    signal WP_ADC_SCLK    : std_logic;   --
    signal WP_ADC_DOUT0   : std_logic := '0';   --
    signal WP_ADC_DOUT1   : std_logic := '0';   --
    signal ZYNQ_TXD       : std_logic; 
    signal ZYNQ_RXD       : std_logic;   --
    signal ZYNQ_CTS       : std_logic;   --
    signal ZYNQ_RTS       : std_logic;   --
    signal COM_ADC_D      : std_logic_vector (13 downto 0);   --
    signal COM_ADC_CSBn   : std_logic;   --
    signal COM_ADC_SCLK   : std_logic; 
    signal COM_ADC_SDIO   : std_logic;   --
    signal COM_ADC_CLK_P  : std_logic;   --
    signal COM_ADC_CLK_N  : std_logic;   --
    signal D00_MOSI       : std_logic;   --
    signal D01_DIN        : std_logic;   --
    signal D02            : std_logic;   --
    signal D03            : std_logic;   --
    signal FLASH_CSn      : std_logic; 
    signal QOSC_CLK       : std_logic := '0';   --
    signal nLED_ENA       : std_logic;   --
    signal nRX_LED        : std_logic;   --
    signal nTX_LED        : std_logic;   --
    signal nRX_LED_EXT    : std_logic;   --
    signal nTX_LED_EXT    : std_logic;   --
    signal COM_DAC_DB     : std_logic_vector (11 downto 0); 
    signal COM_DAC_CLOCK  : std_logic;   --
    signal COM_DAC_SLEEP  : std_logic;   --
    signal USB1_TXD       : std_logic;   --
    signal USB1_RXD       : std_logic;   --
    signal USB1_CTS       : std_logic;   --
    signal USB1_RTS       : std_logic;   --
    signal nCBL_PLGD      : std_logic := '0'; --
    signal ICM_ID         : std_logic;   --
    signal TEST_IO_0      : std_logic;   --
    signal TEST_IO_1      : std_logic;   --
    signal TEST_IO_2      : std_logic;   --
    signal TEST_IO_3      : std_logic;   --
    signal TERM_FPGA      : std_logic;   --
 
  component uart is port -- to emulate the USB to UART bridge 
   (
    clk                : in std_logic;
    reset              : in std_logic;

    tx_data_valid      : in  std_logic;                     -- handshake signal with tx_ack
    tx_data_in         : in  std_logic_vector (7 downto 0); -- parallel tx data input
    tx_busy            : out std_logic;                     -- busy with sending a byte
    tx_ack             : out std_logic;                     -- handshake signal with tx_data_valid
    tx_out             : out std_logic;                     -- serial data output

    rx_in              : in  std_logic;                     -- serial data input
    rx_data_out        : out std_logic_vector (7 downto 0); -- parallel rx data output
    rx_data_valid      : out std_logic                      -- single clock length pulse
    );
   end component uart;

   
 -- ext. uart, inputs
  signal uart_clk      : std_logic := '0';
  signal uart_reset    : std_logic                     := 'U';
  signal tx_data_valid : std_logic                     := '0';
  signal rx_in         : std_logic;
  signal tx_data_in    : std_logic_vector (7 downto 0) := (others => 'U');

-- ext. uart, outputs
  signal tx_ack        : std_logic := '0';
  signal tx_out        : std_logic := '0';
  signal tx_busy       : std_logic;
  signal rx_data_out   : std_logic_vector (7 downto 0) := (others => 'U');
  signal rx_data_valid : std_logic;
 
  constant uart_clk_period : time := 33.33 ns; -- 30 MHz system clock
  constant qosc_clk_period : time := 50 ns;    -- 20 MHz oscillator
   

begin

  uart_clk  <= not uart_clk after uart_clk_period / 2;
  QOSC_CLK  <= not QOSC_CLK after qosc_clk_period / 2;

  USB1_TXD   <= tx_out;  -- external UART connected for serilization / deserialization
  rx_in      <= USB1_RXD;  

	
  ICM_comms_test: process
  	
--	 variable message     : string (1 to message_len) := "  DAC  7  0 5 1234  " & cr & lf;

 --   constant TEST_STRG : string := "Started : ""Generate Programming File""." &cr&lf;
 --  constant TEST_STRG1 : string := "123456789" &cr&lf;
    constant TEST_STRG1 : string := "123";
   
--    constant TEST_STRG2 : string := "if (eof_rcvd = '1')"&cr&lf; --or (no_edge_ct = NO_EDGES_TIME_OUT) or (no_comma_ct = NO_COMMA_TIME_OUT) then";
   constant TEST_STRG2 : string := "ab";
    
  	variable message_len1 : integer;
  	variable message_len2 : integer;	
  	variable i           : integer;
	variable char        : character;
	variable char_ascii  : integer range 0 to 255;
		   
	begin
  
      TEST_IO_0 <= '0'; -- com_thr_adj(0)
      TEST_IO_3 <= '0'; -- com_thr_adj(0)
    
    
      uart_reset <= '1';
      wait for uart_clk_period * 100; -- 3.3us
      uart_reset <= '0';
 	  
--    wait until (reset = '1');
--    wait until (reset = '0');
    
	   wait for uart_clk_period * 200;

      message_len1 := TEST_STRG1'length;
      tx_data_valid <= '0';
	    send_test_strg1: for i in 1 to message_len1 loop
	       -- conversion to ASCII code by getting the position of char
	       -- in the ASCII table "CHARACTER" 
	       char          := TEST_STRG1(i);
	       char_ascii    := CHARACTER'pos(char);
	       tx_data_in       <= std_logic_vector( to_unsigned ( char_ascii, 8));
	       tx_data_valid    <= '1';
	       wait until tx_ack = '1';
	       wait for uart_clk_period *1; --
	        tx_data_valid   <= '0'; 
	    end loop send_test_strg1;
      wait for uart_clk_period * 10000;  
       
  
     message_len2 := TEST_STRG2'length;
	    send_test_strg2: for i in 1 to message_len2 loop
	       -- conversion to ASCII code by getting the position of char
	       -- in the ASCII table "CHARACTER" 
	       char          := TEST_STRG2(i);
	       char_ascii    := CHARACTER'pos(char);
	       tx_data_in    <= std_logic_vector( to_unsigned ( char_ascii, 8));
         tx_data_valid <= '1';
	       wait until tx_ack = '1';
         tx_data_valid <= '0';
	       wait for uart_clk_period *1; -- 
	    end loop send_test_strg2;
      wait for uart_clk_period * 100;   

   wait;
  	    
	end process ICM_comms_test;


	    
-- instantiate the units under test (uut)

uart_inst: uart port map
    (
      reset      => uart_reset,
      clk        => uart_clk,
      tx_data_valid   => tx_data_valid,
      tx_data_in => tx_data_in,
      tx_busy    => tx_busy,
      tx_ack     => tx_ack,
      tx_out     => tx_out,
      rx_in      => rx_in,
      rx_data_out   => rx_data_out,
      rx_data_valid => rx_data_valid
     );

  COM_ADC_D <= COM_DAC_DB & B"00";
 -- COM_ADC_D <= B"00" & X"000";


  mfh_icm_top_01_port_map: component mfh_icm_top_01
   port map
   (
     ICM_CLOCK_P     =>  ICM_CLOCK_P      ,
     ICM_CLOCK_N     =>  ICM_CLOCK_N      ,
     ICM_IRIGB_P     =>  ICM_IRIGB_P    ,
     ICM_IRIGB_N     =>  ICM_IRIGB_N    ,
     ICM_IRIGB_LOC   =>  ICM_IRIGB_LOC  ,
     ICM_nMASTER     =>  ICM_nMASTER    ,
     WP_nPON         =>  WP_nPON        ,
     WP_ADC_nCS0     =>  WP_ADC_nCS0    ,
     WP_ADC_nCS1     =>  WP_ADC_nCS1    ,
     WP_ADC_SCLK     =>  WP_ADC_SCLK    ,
     WP_ADC_DOUT0    =>  WP_ADC_DOUT0   ,
     WP_ADC_DOUT1    =>  WP_ADC_DOUT1   ,
     ZYNQ_TXD        =>  ZYNQ_TXD       ,
     ZYNQ_RXD        =>  ZYNQ_RXD       ,
     ZYNQ_CTS        =>  ZYNQ_CTS       ,
     ZYNQ_RTS        =>  ZYNQ_RTS       ,
     COM_ADC_D       =>  COM_ADC_D      ,
     COM_ADC_CSBn    =>  COM_ADC_CSBn   ,
     COM_ADC_SCLK    =>  COM_ADC_SCLK   ,
     COM_ADC_SDIO    =>  COM_ADC_SDIO   ,
     COM_ADC_CLK_P   =>  COM_ADC_CLK_P  ,
     COM_ADC_CLK_N   =>  COM_ADC_CLK_N  ,
     D00_MOSI        =>  D00_MOSI       ,
     D01_DIN         =>  D01_DIN        ,
     D02             =>  D02            ,
     D03             =>  D03            ,
     FLASH_CSn       =>  FLASH_CSn      ,
     QOSC_CLK        =>  QOSC_CLK       ,
     nLED_ENA        =>  nLED_ENA       ,
     nRX_LED         =>  nRX_LED        ,
     nTX_LED         =>  nTX_LED        ,
     nRX_LED_EXT     =>  nRX_LED_EXT    ,
     nTX_LED_EXT     =>  nTX_LED_EXT    ,
     COM_DAC_DB      =>  COM_DAC_DB     ,
     COM_DAC_CLOCK   =>  COM_DAC_CLOCK  ,
     COM_DAC_SLEEP   =>  COM_DAC_SLEEP  ,
     USB1_TXD        =>  USB1_TXD       ,
     USB1_RXD        =>  USB1_RXD       ,
     USB1_CTS        =>  USB1_CTS       ,
     USB1_RTS        =>  USB1_RTS       ,
     nCBL_PLGD       =>  nCBL_PLGD      ,
     ICM_ID          =>  ICM_ID         ,
     TEST_IO_0       =>  TEST_IO_0      ,
     TEST_IO_1       =>  TEST_IO_1      ,
     TEST_IO_2       =>  TEST_IO_2      ,
     TEST_IO_3       =>  TEST_IO_3      ,
     TERM_FPGA       =>  TERM_FPGA      
    );
  
 --  TEST_IO12     <= TEST_IO13;
--  (TEST_IO7, TEST_IO5, TEST_IO3, TEST_IO1) <= B"1111"; -- baudrate setting

end architecture behavior;
  